;************************************************************************
;
;    Programm:     4-Bit Display Ansteuerung
;                  steuert Display mit dem HD44780 Chip
;    Datum:        01.08.05 MM  Grundversion
;    Angespasst:   01.08.05 MM  ...
;
;    SYSTEM:       89C2051 MCS51  2KB code memory
;    SYSTEM-Info:  
;
;************************************************************************
;
;
; Einzufgen ins Hauptprogramm:
; LCD_RS          EQU P1.0        'RS Leitung des LCD
; LCD_RW          EQU P1.1        'RW Leitung des LCD
; LCD_ENABLE      EQU P1.2        'Enable Leitung des LCD
; LCD_D4          EQU P1.4        'D4 Leitung des LCD
; LCD_D5          EQU P1.5        'D5 Leitung des LCD
; LCD_D6          EQU P1.6        'D6 Leitung des LCD
; LCD_D7          EQU P1.7        'D7 Leitung des LCD
; INCLUDE 	  lcd_4bit.asm    'Einbindung der LCD Routinen
; LCD_4BIT: LCD_Init, LCD_Befehl, LCD_schreiben, LCD_prints
;
; Zur Verfgung stehende Routinen:
; 1. LCD_Init 	-	Initiert das Display.
; 2. LCD_Befehl	-	Sendet den Befehl im Akku an den HD44 Kontroller
; 3. LCD_schreiben -	Sendet das Zeichen im Akku an das Display
; 4. LCD_prints	-	Sendet eine Zeichenfolge (DPTR) bis zu einer 0 (siehe Bsp.)
; 5. LCD_HEX -          Sendet A als HEX aufs LCD
;
; Aufrufe Beispiel:
; - CALL LCD_Init		'LC-Display Ansteuerung
;
; - MOV DPTR,#Zeile1		'Pointer auf die Zeichenkette setzen
; - CALL LCD_prints		'Zeichenkette mit 0 Terminierung ausgeben
;
; - MOV A,#">"			'Schreibt ein Prompt (grsser als)
; - CALL LCD_Schreiben
;
; - MOV A,#LCD_Zeile1 		'Sprung zur ersten Stelle der ersten Zeile
; - CALL LCD_Befehl	
; - ;Liste der Befehle:LCD_Zeile1, LCD_Zeile2, LCD_Clear, LCD_homereturn		EQU !00000010	'Adresszhler auf 00h
; - ;LCD_Modus, LCD_on, LCD_links, LCD_rechts, LCD_interface
;
;************************************************************************
INCLUDE 89C4051.MC 
;
;
;Grundbefehle LCD-Controller:
LCD_Zeile1		EQU !10000000	'Adresse der 1.Zeile,1.Stelle
LCD_Zeile2		EQU !11000000	'Adresse der 2.Zeile,1.Stelle
LCD_Clear		EQU !00000001	'Lscht die Anzeige (Adresse 00 = 10000000b, AutoIncrement)
LCD_homereturn	EQU !00000010	'Adresszhler auf 00h
LCD_Modus		EQU !00000110	'Betrieb Increment (ohne Shift)
LCD_on		EQU !00001100	'Display-Ausgabe An (Kein Coursor, Kein Blinken)
LCD_links		EQU !00011100	'Bewegt Displayinhalt um 1 nach links
LCD_rechts		EQU !00011000	'Bewegt Displayinhalt um 1 nach rechts
LCD_interface	EQU !00101000	'Betreibsmodus Interface (zwei Zeilen, 4Bit)  


; ## Initialisiert den LCD Controller HD 44xxx des Display
;		ENTRY: NO
;		EXIT: NO
;		Destroy: A,R7

LCD_Init:	CLR LCD_ENABLE

		MOV R7,#248			  ;warten, bis sich das Display initialisiert hat (1,7ms)
wait_s0:   	NOP
		NOP
		NOP
		NOP
		DJNZ R7,wait_s0		'R7--: If (R7 > 0) _wait_s0

                MOV A,#LCD_Interface    ' Betreibsmodus Interface (zwei Zeilen, 4Bit)  
	        CALL LCD_Befehl
        	MOV A,#LCD_on	  	'Aktivierung der Anzeige
		CALL LCD_Befehl
	  	MOV A,#LCD_Zeile1 	'Sprung zur ersten Stelle der ersten Zeile
		CALL LCD_Befehl	
		RET


; ## Sendet Befehl des Akku an das Display
;		ENTRY: A
;		EXIT: NO
;		Destroy: -
LCD_Befehl:     CLR LCD_RS		'Setzt den "Befehls-Modus"

; ## 		Sendet A an das Display
;		ENTRY: A
;		EXIT: NO
;		Destroy: -
LCD_Schreiben:  CLR LCD_RW		'Setzt den "Schreib-Modus"
            	mov   c,acc.4		'Hi Nibble Bit 4. OutPut
		mov   LCD_D4,c
           	mov   c,acc.5		'Hi Nibble Bit 5. OutPut
            	mov   LCD_D5,c
            	mov   c,acc.6		'Hi Nibble Bit 6. OutPut
            	mov   LCD_D6,c
            	mov   c,acc.7		'Hi Nibble Bit 7. OutPut
            	mov   LCD_D7,c

		SETB LCD_ENABLE		'Hier wird das Display zum Lesen dieser Werte aufgefordert,		
		CLR LCD_ENABLE		'sollte 1s nicht ausreichen, dann msst Ihr zw. SETB u. CLR einige NOP setzen

            	mov   c,acc.0		'An dieser Stelle werden jetzt die zweiten 4 Bit gesendet.
            	mov   LCD_D4,c		'Lo Nibble Bit 4. OutPut
            	mov   c,acc.1
            	mov   LCD_D5,c		'Lo Nibble Bit 4. OutPut
            	mov   c,acc.2
            	mov   LCD_D6,c		'Lo Nibble Bit 4. OutPut
            	mov   c,acc.3
            	mov   LCD_D7,c		'Lo Nibble Bit 4. OutPut

		SETB LCD_ENABLE		'Hier wird das Display zum Lesen dieser Werte aufgefordert,		
		CLR LCD_ENABLE		'sollte 1s nicht ausreichen, dann msst Ihr zw. SETB u. CLR einige NOP setzen

		NOP			'kurze Pause nach dem Low / Hi Nibble geschrieben Sind
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP			'wird vor dem nchsten Senden eines Bytes immer abgewartet,
		NOP			'ob das Display Bereit ist. Ist es nicht der Fall, dann
		NOP			'bleibt er in der Schleife dis das Display verfgbar ist.

		CLR LCD_RS		'Select Register 0
		SETB LCD_RW		'Lese Modus
		mov R7,#0		'Timout wenn kein Display angeschlossen ist
busy:  		DJNZ R7,NoBusyTimeOut	'R7--: If (R7 > 0) 
		JMP BusyTimeOut		'Es Gibt halt kein DJZ Befehl ?? beim MCS51
NoBusyTimeOut:	SETB LCD_ENABLE
		MOV C,LCD_D7
		CLR LCD_ENABLE
		SETB LCD_ENABLE		'zweites Bibble Abfragen NOTWENDIG obwohls nicht gebraucht wird
		CLR LCD_ENABLE		'zweites Bibble Abfragen NOTWENDIG obwohls nicht gebraucht wird
		NOP			'kurze Pause nach dem Low / Hi Nibble geschrieben Sind
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		JC Busy
BusyTimeOut:	SETB LCD_RS		'Vorgabe gelscht da zwei einsprnge Befehl / Schreiben!!
		RET


; ## 		Sendet A in HEX an das Display
;		ENTRY: A
;		EXIT: NO
;		Destroy: -
LCD_Hex:	PUSH  ACC                ; Akku retten
		SWAP  A                  ; High Nibble zuerst
		CALL  LCD_Hex2Asc      ; konvertieren
		POP   ACC                ; Akku zurck 
		CALL  LCD_Hex2Asc      ; Low Nibble
		RET

LCD_Hex2Asc:	anl	a,#0fh		; Unteres Nibbel Abmskieren
		add	a,#246		; bertrag im Carry
		jnc	LCD_Hex2Asc2	; grsser 9
		add	a,#07		; 'A'-'9'
LCD_Hex2Asc2:	add	a,#58		; '9'+1
		CALL  LCD_Schreiben      
		RET




; ## Ausgabe eines mit 0 terminierten Strings aus DPTR
;		ENTRY: DPTR
;		EXIT: NO
;		Destroy: A,RS0,RS1,R7,DPTR
LCD_prints:	PUSH A
		PUSH psw
		PUSH dpl
		PUSH dph
		CLR rs0
		CLR rs1
LCD_prints_anf: CLR A
		MOVC A,@A+DPTR
		JZ LCD_prints_weit	'Ist das letzte Byte als 0 erkannt, endet die Ausgabe
		INC DPTR
		CALL LCD_Schreiben
		JMP LCD_prints_anf
LCD_prints_weit:POP dph
		POP dpl
		POP psw
		POP A
		RET       
